//============================================================================
// 632 / 634 WinTest Code.
// Copyright 2001, Crystalfontz America, Inc. Written by Brent A. Crosby
// www.crystalfontz.com, brent@crystalfontz.com
//============================================================================
#include  <stdlib.h>
#include  <termios.h>
#include  <unistd.h>
#include  <fcntl.h>
#include  <errno.h>
#include  <ctype.h>
#include  <stdio.h>
#include  "typedefs.h"
#include  "serial.h"

#define FALSE   0
#define TRUE    1

// com port handle
int handle;

// data buffering
#define RECEIVEBUFFERSIZE 4096
ubyte SerialReceiveBuffer[RECEIVEBUFFERSIZE];
dword ReceiveBufferHead;
dword ReceiveBufferTail;
dword ReceiveBufferTailPeek;

//------------------------------------------------------------------------------
int Serial_Init(char *devname, int baud_rate)
  {
  int
    brate;
  struct
    termios term; 

  //open device
  handle = open(devname, O_RDWR | O_NOCTTY | O_NONBLOCK);

  if(handle <= 0)
    {
      	printf("Serial_Init:: Open() failed\n");
	return(1);
    }

  //get baud rate constant from numeric value
  switch (baud_rate)
  {
    case 1200:
      brate=B1200;
      break;
    case 2400:
      brate=B2400;
      break;
    case 4800:
      brate=B4800;
      break;
    case 9600:
      brate=B9600;
      break;
    case 19200:
      brate=B19200;
      break;
    default:
      printf("Serial_Init:: Invalid baud rate: %d (must be 1200, 2400, 4800, 9600, or 19200)\n", baud_rate);
      return(2);
  }
  //get device struct
  if(tcgetattr(handle, &term) != 0)
    {
    printf("Serial_Init:: tcgetattr() failed\n");
    return(3);
    }

  //input modes 
  term.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|INPCK|ISTRIP|INLCR|IGNCR|ICRNL
                  |IXON|IXOFF);
  term.c_iflag |= IGNPAR;

  //output modes 
  term.c_oflag &= ~(OPOST|ONLCR|OCRNL|ONOCR|ONLRET|OFILL
                  |OFDEL|NLDLY|CRDLY|TABDLY|BSDLY|VTDLY|FFDLY);
  term.c_oflag |= NL0|CR0|TAB0|BS0|VT0|FF0;

  //control modes
  term.c_cflag &= ~(CSIZE|PARENB|PARODD|HUPCL|CRTSCTS);
  term.c_cflag |= CREAD|CS8|CSTOPB|CLOCAL;

  //local modes 
  term.c_lflag &= ~(ISIG|ICANON|IEXTEN|ECHO);
  term.c_lflag |= NOFLSH;

  //set baud rate
  cfsetospeed(&term, brate);
  cfsetispeed(&term, brate);

  //set new device settings
  if(tcsetattr(handle, TCSANOW, &term)  != 0)
    {
    printf("Serial_Init:: tcsetattr() failed\n");
    return(4);
    }

  printf("Serial_Init:: success\n");
  return(0);
  }
//------------------------------------------------------------------------------
void Uninit_Serial()
  {
  close(handle);
  handle=0;
  }
//------------------------------------------------------------------------------
void SendByte(unsigned char datum)
  {
  dword
    bytes_written;
  bytes_written=0;

  if(handle)
    if(write(handle, &datum, 1) != 1)
      printf("SendByte() failed\n");
  }
//------------------------------------------------------------------------------
void SendString(char *data)
  {
  while(*data)
    SendByte(*data++);
  }
//------------------------------------------------------------------------------
